==> dune-fem-2.4.1.tar.gz: dune-fem-2.4.1/dune/fem/test/vtxprojection.cc <==
/**************************************************************************
 
  The dune-fem module is a module of DUNE (see www.dune-project.org).
  It is based on the dune-grid interface library 
  extending the grid interface by a number of discretization algorithms
  for solving non-linear systems of partial differential equations.

  Copyright (C) 2003 - 2015 Robert Kloefkorn
  Copyright (C) 2003 - 2010 Mario Ohlberger 
  Copyright (C) 2004 - 2015 Andreas Dedner
  Copyright (C) 2005        Adrian Burri
  Copyright (C) 2005 - 2015 Mirko Kraenkel
  Copyright (C) 2006 - 2015 Christoph Gersbacher
  Copyright (C) 2006 - 2015 Martin Nolte
  Copyright (C) 2011 - 2015 Tobias Malkmus
  Copyright (C) 2012 - 2015 Stefan Girke
  Copyright (C) 2013 - 2015 Claus-Justus Heine
  Copyright (C) 2013 - 2014 Janick Gerstenberger
  Copyright (C) 2013        Sven Kaulman
  Copyright (C) 2013        Tom Ranner
  Copyright (C) 2015        Marco Agnese
  Copyright (C) 2015        Martin Alkaemper


  The dune-fem module is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License as 
  published by the Free Software Foundation; either version 2 of 
  the License, or (at your option) any later version.

  The dune-fem module is distributed in the hope that it will be useful, 
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
**************************************************************************/
#undef NDEBUG

#include <config.h>
#include <iostream>

#include <dune/fem/misc/mpimanager.hh>
#include <dune/fem/operator/projection/vtxprojection.hh>
#include <dune/fem/operator/projection/l2projection.hh>
#include <dune/fem/operator/lagrangeinterpolation.hh>

#if defined USE_BLOCKVECTORFUNCTION
#include <dune/fem/function/blockvectorfunction.hh>
#elif defined USE_VECTORFUNCTION
#include <dune/fem/storage/vector.hh>
#include <dune/fem/function/vectorfunction.hh>
#else
#include <dune/fem/function/adaptivefunction.hh>
#endif

#include "testgrid.hh"
#include "dfspace.hh"
// #include "vtxl2projection.hh"
#include "exactsolution.hh"
#include <dune/fem/gridpart/adaptiveleafgridpart.hh>

using namespace Dune;
using namespace Fem;

typedef GridSelector::GridType MyGridType;
// typedef HierarchicGridPart< MyGridType > GridPartType;
typedef Fem::AdaptiveLeafGridPart< MyGridType > GridPartType;

typedef TestFunctionSpace FunctionSpaceType;
typedef TestDiscreteFunctionSpace< GridPartType > DiscreteFunctionSpaceType;

#if defined USE_BLOCKVECTORFUNCTION
typedef Fem :: ISTLBlockVectorDiscreteFunction< DiscreteFunctionSpaceType >
  DiscreteFunctionType;
#elif defined USE_VECTORFUNCTION
typedef Fem :: ManagedDiscreteFunction
  < Fem::VectorDiscreteFunction
    < DiscreteFunctionSpaceType,
      Fem::DynamicVector< FunctionSpaceType :: RangeFieldType > > >
  DiscreteFunctionType;
#else
typedef Fem :: AdaptiveDiscreteFunction< DiscreteFunctionSpaceType >
  DiscreteFunctionType;
#endif

typedef ExactSolution< FunctionSpaceType > ExactSolutionType;

#include <dune/fem/io/file/vtkio.hh>

// main program
int main(int argc, char ** argv)
{
  MPIManager :: initialize( argc, argv );
  try
  {
    MyGridType &grid = TestGrid :: grid();
    // const int step = TestGrid :: refineStepsForHalf();
    // grid.globalRefine( 2*step );

    GridPartType gridPart( grid );
    DiscreteFunctionSpaceType discreteFunctionSpace( gridPart ); // DG
    typedef LagrangeDiscreteFunctionSpace<FunctionSpaceType, GridPartType,
      1,CachingStorage> LDiscreteFunctionSpaceType;
    LDiscreteFunctionSpaceType lagspace(gridPart);
    ExactSolutionType exactSolution;
    typedef AdaptiveDiscreteFunction<LDiscreteFunctionSpaceType> LagrangeFunctionType;

    // perform the L2Projection
    DiscreteFunctionType solution( "solution", discreteFunctionSpace );
    Fem::L2Projection< ExactSolutionType,  DiscreteFunctionType > dgl2;
    dgl2( exactSolution, solution );

    LagrangeFunctionType contSolution("contSolution",lagspace);
    VtxProjection< DiscreteFunctionType,LagrangeFunctionType > projection;
    projection(solution,contSolution);

    LagrangeFunctionType lagrangeSolution("lagrangeSolution",lagspace);
    Fem::LagrangeInterpolation< ExactSolutionType, LagrangeFunctionType >
      :: interpolateFunction( exactSolution, lagrangeSolution );
    #if 0
    LagrangeType lagrangeContSolution("lagrnageContSolution",lagspace);
    VtxProjection<double,double,LagrangeType,LagrangeType> projection2;
    projection2(lagrangeSolution,lagrangeContSolution);

    // output to vtk file
    VTKIO<GridPartType> vtkWriter(gridPart);
    vtkWriter.addCellData(solution);
    vtkWriter.addVertexData(contSolution);
    lagrangeSolution -= lagrangeContSolution;
    vtkWriter.addVertexData(lagrangeSolution);
    vtkWriter.addVertexData(lagrangeContSolution);
    vtkWriter.pwrite("vtxprojection",
		     Parameter::commonOutputPath().c_str(),".",
		     Dune::VTKOptions::ascii);

#endif
    return 0;
  }
  catch( Exception e )
  {
    std :: cerr << e.what() << std :: endl;
    return 1;
  }
}

==> dune-fem-2.9.0.dev20220404.tar.gz: dune-fem-2.9.0.dev20220404/dune/fem/test/vtxprojection.cc <==
#undef NDEBUG

#include <config.h>
#include <iostream>

#include <dune/grid/common/rangegenerators.hh>

#include <dune/fem/misc/mpimanager.hh>
#include <dune/fem/operator/projection/vtxprojection.hh>
#include <dune/fem/space/common/interpolate.hh>

#if defined USE_BLOCKVECTORFUNCTION
#include <dune/fem/function/blockvectorfunction.hh>
#elif defined USE_VECTORFUNCTION
#include <dune/common/dynvector.hh>
#include <dune/fem/function/vectorfunction.hh>
#else
#include <dune/fem/function/adaptivefunction.hh>
#endif

#include "testgrid.hh"
#include "dfspace.hh"
// #include "vtxl2projection.hh"
#include "exactsolution.hh"
#include <dune/fem/gridpart/adaptiveleafgridpart.hh>
#include <dune/fem/function/common/gridfunctionadapter.hh>

using namespace Dune;
using namespace Fem;

typedef GridSelector::GridType MyGridType;
// typedef HierarchicGridPart< MyGridType > GridPartType;
typedef Fem::AdaptiveLeafGridPart< MyGridType > GridPartType;

typedef TestFunctionSpace FunctionSpaceType;
typedef TestDiscreteFunctionSpace< GridPartType > DiscreteFunctionSpaceType;

#if defined USE_BLOCKVECTORFUNCTION
typedef Fem :: ISTLBlockVectorDiscreteFunction< DiscreteFunctionSpaceType >
  DiscreteFunctionType;
#elif defined USE_VECTORFUNCTION
typedef Fem :: ManagedDiscreteFunction
  < Fem::VectorDiscreteFunction
    < DiscreteFunctionSpaceType,
      Dune::DynamicVector< FunctionSpaceType :: RangeFieldType > > >
  DiscreteFunctionType;
#else
typedef Fem :: AdaptiveDiscreteFunction< DiscreteFunctionSpaceType >
  DiscreteFunctionType;
#endif

typedef ExactSolution< FunctionSpaceType > ExactSolutionType;

#include <dune/fem/io/file/vtkio.hh>

// main program
int main(int argc, char ** argv)
{
  MPIManager :: initialize( argc, argv );
  try
  {
    MyGridType &grid = TestGrid :: grid();
    const int step = TestGrid :: refineStepsForHalf();
    grid.globalRefine( 2*step );

    for( const auto& entity : elements(grid.leafGridView()) )
    {
      std::remove_reference_t<decltype(entity)>::Geometry::GlobalCoordinate x(0.1);
      x -= entity.geometry().center();
      if( x.two_norm() > 0.3 && x.two_norm() < 0.4 )
        grid.mark( 1, entity );
    }
    grid.preAdapt();
    grid.adapt();
    grid.postAdapt();


    GridPartType gridPart( grid );
    DiscreteFunctionSpaceType discreteFunctionSpace( gridPart ); // DG

#if HAVE_DUNE_LOCALFUNCTIONS
    typedef LagrangeSpace<FunctionSpaceType, GridPartType>
      LDiscreteFunctionSpaceType;
    LDiscreteFunctionSpaceType lagspace(gridPart,1);
#else
    typedef LagrangeDiscreteFunctionSpace<FunctionSpaceType, GridPartType, 1>
      LDiscreteFunctionSpaceType;
    LDiscreteFunctionSpaceType lagspace(gridPart);
#endif

    ExactSolutionType exactSolution;
    typedef AdaptiveDiscreteFunction<LDiscreteFunctionSpaceType> LagrangeFunctionType;

    // perform the L2Projection
    DiscreteFunctionType solution( "solution", discreteFunctionSpace );
    interpolate( exactSolution, solution );

    LagrangeFunctionType contSolution("contSolution",lagspace);
    VtxProjection< DiscreteFunctionType,LagrangeFunctionType > projection;
    projection(solution,contSolution);

    LagrangeFunctionType lagrangeSolution("lagrangeSolution",lagspace);
    interpolate( gridFunctionAdapter( exactSolution, gridPart, lagspace.order()+2 ), lagrangeSolution );
#if 0
    LagrangeFunctionType testSolution("test",lagspace);
    VtxProjection<LagrangeFunctionType,LagrangeFunctionType> projection2;
    projection2(lagrangeSolution,testSolution);

    // output to vtk file
    VTKIO<GridPartType> vtkWriter(gridPart);
    vtkWriter.addCellData(solution);
    vtkWriter.addVertexData(contSolution);
    testSolution -= lagrangeSolution;
    vtkWriter.addVertexData(lagrangeSolution);
    vtkWriter.addVertexData(testSolution);
    vtkWriter.pwrite("vtxprojection", Parameter::commonOutputPath().c_str(),".");
#endif
    return 0;
  }
  catch( const Exception& e )
  {
    std :: cerr << e.what() << std :: endl;
    return 1;
  }
}
